{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit sharedFoldersUnit;

interface

uses
	classes, sysUtils, iniFiles, dateUtils, basicDefsUnit;

const
	{ shared folder type }
	sftSharedDir = 0;
	sftIncoming = 1;
	sftCategory = 2;

type
	tSharedFolders = class( tStringList )
		protected
			fLoadedIncoming: boolean;
			fLoadedSharedDir: boolean;
			fLoadedCategory: boolean;
			fDateIncoming: tFileDate;
			fDateSharedDir: tFileDate;
			fDateCategory: tFileDate;
			{ 15 feb 09: emule0.49c: Share/unshare single files >>> }
			fIncludedFiles: tStringList;
			fExcludedFiles: tStringList;
			fLoadedSharedFiles: boolean;
			fDateSharedFiles: tFileDate;
			function getNeedUpdateSharedFiles: boolean;
			{ <<< 15 feb 09: emule0.49c: Share/unshare single files }
			function getNeedUpdateIncoming: boolean;
			function getNeedUpdateSharedDir: boolean;
			function getNeedUpdateCategory: boolean;
		public
			constructor create;
			procedure updateIncoming;
			procedure updateSharedDir;
			procedure updateCategory;
			{ 15 feb 09: emule0.49c: Share/unshare single files >>> }
			destructor destroy; override;
			procedure updateSharedFiles;
			property includedFiles: tStringList read fIncludedFiles;
			property excludedFiles: tStringList read fExcludedFiles;
			property loadedSharedFiles: boolean read fLoadedSharedFiles;
			property dateSharedFiles: tFileDate read fDateSharedFiles;
			property needUpdateSharedFiles: boolean read getNeedUpdateSharedFiles;
			{ <<< 15 feb 09: emule0.49c: Share/unshare single files }
			property loadedIncoming: boolean read fLoadedIncoming;
			property loadedSharedDir: boolean read fLoadedSharedDir;
			property loadedCategory: boolean read fLoadedCategory;
			property dateIncoming: tFileDate read fDateIncoming;
			property dateSharedDir: tFileDate read fDateSharedDir;
			property dateCategory: tFileDate read fDateCategory;
			property needUpdateIncoming: boolean read getNeedUpdateIncoming;
			property needUpdateSharedDir: boolean read getNeedUpdateSharedDir;
			property needUpdateCategory: boolean read getNeedUpdateCategory;
		end;

implementation

constructor tSharedFolders.create;
begin
	inherited create;
	caseSensitive := false;
	duplicates := dupIgnore;
	sorted := true;
	fLoadedIncoming := false;
	fLoadedSharedDir := false;
	fLoadedCategory := false;
	fDateIncoming := -1;
	fDateSharedDir := -1;
	fDateCategory := -1;
	{ 15 feb 09: emule0.49c: Share/unshare single files >>> }
	fIncludedFiles := tStringList.create;
	fIncludedFiles.sorted := true;
	fIncludedFiles.caseSensitive := false;
	fIncludedFiles.duplicates := dupIgnore;
	fExcludedFiles := tStringList.create;
	fExcludedFiles.sorted := true;
	fExcludedFiles.caseSensitive := false;
	fExcludedFiles.duplicates := dupIgnore;
	fLoadedSharedFiles := false;
	fDateSharedFiles := -1;
	{ <<< 15 feb 09: emule0.49c: Share/unshare single files }
end;

{ 15 feb 09: emule0.49c: Share/unshare single files >>> }
destructor tSharedFolders.destroy;
begin
	fIncludedFiles.free;
	fExcludedFiles.free;
	inherited destroy;
end;
{ <<< 15 feb 09: emule0.49c: Share/unshare single files }

procedure tSharedFolders.updateIncoming;
var
	i: integer;
	f: tIniFile;
	path, s: string;
begin
	s := '';
	path := pnEmuleConfig + fnPreferencesIni;
	try
		f := tIniFile.create( path );
		try
			s := f.readString( isEmule, ikIncoming, '' );
			finally f.free;
			end;
		except ;
		end;
	fLoadedIncoming := (s <> '');
	if fLoadedIncoming then
		begin
		fDateIncoming := fileAge( path );
		for i := pred( count ) downTo 0 do
			if objects[ i ] = tObject( sftIncoming ) then
				begin
				delete( i );
				break;
				end;
		addObject( includeTrailingPathDelimiter( s ), tObject( sftIncoming ) );
		end;
end;

procedure tSharedFolders.updateSharedDir;
var
	path, s: string;
	sl: tStringList;
	i: integer;
begin
	path := pnEmuleConfig + fnSharedDirDat;
	if fileExists( path ) then
		begin
		fLoadedSharedDir := unicodeFileToString( path, s );
		if fLoadedSharedDir then fDateSharedDir := fileAge( path );
		end
	else
		begin
		fLoadedSharedDir := true;
		fDateSharedDir := -1;
		end;
	if fLoadedSharedDir then
		begin
		sl := tStringList.create;
		try
			sl.text := s;
			for i := pred( count ) downTo 0 do
				if objects[ i ] = tObject( sftSharedDir ) then delete( i );
			for i := 0 to pred( sl.count ) do
				addObject( includeTrailingPathDelimiter( sl[ i ] ), tObject( sftSharedDir ) );
			finally sl.free;
			end;
		end;
end;

procedure tSharedFolders.updateCategory;
var
	n, i: integer;
	f: tIniFile;
	path, s: string;
	sl: tStringList;
begin
	sl := tStringList.create;
	fLoadedCategory := false;
	try
		path := pnEmuleConfig + fnCategoryIni;
		if fileExists( path ) then
			begin
			try
				f := tIniFile.create( path );
				try
					n := f.readInteger( 'General', 'Count', 0 );
					for i := 0 to n do
						begin
						s := f.readString( 'Cat#' + intToStr( i ), 'Incoming', '' );
						if s <> '' then
							sl.add( includeTrailingPathDelimiter( s ) );
						end;
					fLoadedCategory := true;
					finally f.free;
					end;
				except ;
				end;
			if fLoadedCategory then fDateCategory := fileAge( path );
			end
		else
			begin
			fLoadedCategory := true;
			fDateCategory := -1;
			end;
		if fLoadedCategory then
			begin
			for i := pred( count ) downTo 0 do
				if objects[ i ] = tObject( sftCategory ) then delete( i );
			for i := 0 to pred( sl.count ) do
				addObject( sl[ i ], tObject( sftCategory ) );
			end;
		finally sl.free;
		end;
end;

{ 15 feb 09: emule0.49c: Share/unshare single files >>> }
procedure tSharedFolders.updateSharedFiles;
var
	path, s: string;
	sl: tStringList;
	i: integer;
begin
	path := pnEmuleConfig + fnSharedFilesDat;
	if fileExists( path ) then
		begin
		fLoadedSharedFiles := unicodeFileToString( path, s );
		if fLoadedSharedFiles then fDateSharedFiles := fileAge( path );
		end
	else
		begin
		fLoadedSharedFiles := true;
		fDateSharedFiles := -1;
		end;
	if fLoadedSharedFiles then
		begin
		fIncludedFiles.clear;
		fExcludedFiles.clear;
		sl := tStringList.create;
		try
			sl.text := s;
			for i := 0 to pred( sl.count ) do
				begin
				s := sl[ i ];
				if s <> '' then
					begin
					if s[ 1 ] = '-' then fExcludedFiles.add( copy( s, 2, maxInt ) )
					else fIncludedFiles.add( s );
					end;
				end;
			finally sl.free;
			end;
		end;
end;
{ <<< 15 feb 09: emule0.49c: Share/unshare single files }

function tSharedFolders.getNeedUpdateIncoming: boolean;
var
	fa: tFileDate;
begin
	fa := fileAge( pnEmuleConfig + fnPreferencesIni );
	result := (not fLoadedIncoming or (fDateIncoming <> fa)) and (millisecondsBetween( now, fileDateToDateTime( fa ) ) > delayAfterDateChange);
end;

function tSharedFolders.getNeedUpdateSharedDir: boolean;
var
	fa: integer;
begin
	fa := fileAge( pnEmuleConfig + fnSharedDirDat );
	if fa = -1 then result := not fLoadedSharedDir or (fDateSharedDir <> -1) and (millisecondsBetween( now, fileDateToDateTime( fDateSharedDir ) ) > delayAfterDateChange)
	else result := (not fLoadedSharedDir or (fDateSharedDir <> fa)) and (millisecondsBetween( now, fileDateToDateTime( fa ) ) > delayAfterDateChange);
end;

function tSharedFolders.getNeedUpdateCategory: boolean;
var
	fa: integer;
begin
	fa := fileAge( pnEmuleConfig + fnCategoryIni );
	if fa = -1 then result := not fLoadedCategory or (fDateCategory <> -1) and (millisecondsBetween( now, fileDateToDateTime( fDateCategory ) ) > delayAfterDateChange)
	else result := (not fLoadedCategory or (fDateCategory <> fa)) and (millisecondsBetween( now, fileDateToDateTime( fa ) ) > delayAfterDateChange);
end;

{ 15 feb 09: emule0.49c: Share/unshare single files >>> }
function tSharedFolders.getNeedUpdateSharedFiles: boolean;
var
	fa: integer;
begin
	fa := fileAge( pnEmuleConfig + fnSharedFilesDat );
	if fa = -1 then result := not fLoadedSharedFiles or (fDateSharedFiles <> -1) and (millisecondsBetween( now, fileDateToDateTime( fDateSharedFiles ) ) > delayAfterDateChange)
	else result := (not fLoadedSharedFiles or (fDateSharedFiles <> fa)) and (millisecondsBetween( now, fileDateToDateTime( fa ) ) > delayAfterDateChange);
end;
{ <<< 15 feb 09: emule0.49c: Share/unshare single files }

end.
