{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit logRecorderUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, dateUtils, ExtCtrls, iniFiles, registry, shellApi, Menus, ActnList, StdActns,
  basicDefsUnit, knownMetUnit;

resourceString
	{ 07 may 07: Bug fix: possible infinite loop on log updates -> timeout = 15 failed attempts >>> }
	msgFatalError = 'Unexpected error. Failed to update the logs %d consecutive times';
	{ <<< 07 may 07: Bug fix: possible infinite loop on log updates -> timeout = 15 failed attempts }

const
	wm_trayIcon = wm_app;

const
	fnViewer = 'EFSViewer.exe';

const
	timer_normal = timer_default * 60000;

type
  tMainForm = class(TForm)
	timer: TTimer;
	actions: TActionList;
	trayPopup: TPopupMenu;
	Exit1: TMenuItem;
	aOpen: TAction;
	Open1: TMenuItem;
	aExit: TAction;
	N1: TMenuItem;
	procedure FormCreate(Sender: TObject);
	procedure trayIcon( cmd: dWord );
	procedure FormDestroy(Sender: TObject);
	procedure timerTimer(Sender: TObject);
	procedure aOpenExecute(Sender: TObject);
	procedure aExitExecute(Sender: TObject);
	procedure aOpenUpdate(Sender: TObject);
	procedure wndProc( var message: tMessage ); override;
    procedure aExitUpdate(Sender: TObject);
  protected
	procedure wmTrayIcon( var message: tMessage ); message wm_trayIcon;
  private
  public
  end;

var
	mainForm: tMainForm;

	knownMetChgd: boolean = false;

	closed: boolean = false;
	{ set to true, when the user closes the program.
	the program will actually exit after the logs have been updated }

	suspended: boolean = false;
	{ set to true, when a suspendMsg is received from the viewer;
	reset to false when a resumeMsg is received;
	while in suspended mode, the logs are not written, so the viewer can open them safely with write access }

	timerStart: dWord;
	{ used to restore the timer interval to the remaining period after a resumedMsg has been received }

	trayIconOk: boolean = false;
	{ set to true when the tray icon has been successfully added;
	reset to false when a taskBarCreated message is received;
	while it's false, new attempts to create the tray icon are made in the onTimer method
	(because it can fail on windows startup) }

	{ 07 may 07: Bug fix: possible infinite loop on log updates -> timeout = 15 failed attempts >>> }
	failures: integer = 0;
	{ <<< 07 may 07: Bug fix: possible infinite loop on log updates -> timeout = 15 failed attempts }

implementation

{$R *.dfm}

procedure tMainForm.FormCreate(Sender: TObject);
begin
	{ terminate application if another instance (same folder only) is already running }
	createMutex( nil, false, pChar( mutexPath + mnRecorder ) );
	if (getLastError = error_already_exists) or (getLastError = error_access_denied) then
		halt;

	application.showMainForm := false;

	onErrorLogged := trayIcon;

	logFiles := tLogFiles.create;
	logFiles.load; 

	trayIcon( nim_add );

	registerMessages;

	{ first update = as soon as possible }
	timer.interval := timer_init;
	timerStart := getTickCount;
	timer.enabled := true;
end;

{ 13 may 07: 0.48a support (new folders): Show eMule config folder path instead of eMule executable folder path, and use shell_notifyIcon v5 if available to avoid displaying a truncated path >>> }
type
	pDLLVersionInfo = ^tDLLVersionInfo;
	tDLLVersionInfo = record
	cbSize,
	dwMajorVersion,
	dwMinorVersion,
	dwBuildNumber,
	dwPlatformID: dWord;
	end;

function getDllVersion( const DLLName: string; var DLLVersionInfo: tDLLVersionInfo ): boolean;
type
  	tDLLGetVersion = function( dvi: pDLLVersionInfo ): hResult; stdcall;
var
	hInstDLL: tHandle;
	DLLGetVersion: tDLLGetVersion;
begin
	result := false;
	hInstDLL := loadLibrary( pChar( DLLName ) );
	if (hInstDLL = 0) then exit;
	try
		@DLLGetVersion := getProcAddress( hInstDLL, 'DllGetVersion' );
		if (@DLLGetVersion) = nil then exit;

		zeroMemory( @DLLVersionInfo, sizeOf( DLLVersionInfo ) );
		DLLVersionInfo.cbSize := sizeOf( DLLVersionInfo );
		if DLLGetVersion( @DLLVersionInfo ) <> NOERROR then exit;
		result := true;

		finally freeLibrary( hInstDLL );
		end;
end;

procedure tMainForm.trayIcon( cmd: dWord );
type
	tNotifyIconDataV5 = record
		cbSize: dWord;
		Wnd: hWnd;
		uID: dWord;
		uFlags: dWord;
		uCallbackMessage: dWord;
		hIcon: hIcon;
		szTip: array [0..127] of ansiChar;
		dwState: dWord;
		dwStateMask: dWord;
		szInfo: array [0..255] of ansiChar;
		case integer of
			0: (
				uTimeout: dWord;
				szInfoTitle : array [0..63] of ansiChar;
				dwInfoFlags: dWord;
				guidItem: tGuid
			);
			1: (
				uVersion: dWord
			);
		end;
var
	nid: tNotifyIconDataV5;
	shell32Version: tDLLVersionInfo;
	shell32V5: boolean;
	tipMaxSize: integer;
begin
	zeroMemory( @nid, sizeOf( nid ) );
	with nid do
		begin
		shell32V5 := getDLLVersion( 'shell32.dll', shell32Version ) and (shell32Version.dwMajorVersion >= 5);
		if shell32V5 then
			begin
			cbSize := sizeOf( nid );
			tipMaxSize := pred( sizeOf( szTip ) );
			end
		else
			begin
			cbSize := sizeOf( tNotifyIconData );
			tipMaxSize := 63;
			end;
		wnd := handle;
		uId := 0;
		if (cmd = nim_add) or (cmd = nim_modify) then
			begin
			uFlags := nif_icon or nif_message or nif_tip;
			uCallbackMessage := wm_trayIcon;
			if errorsLogged then hIcon := loadIcon( hInstance, 'TRAYICON_ERROR' )
			else hIcon := loadIcon( hInstance, 'TRAYICON' );
			strLCopy( szTip, pChar( appFullName + #13 + pnEmuleConfig ), tipMaxSize );
			end;
		end;
	trayIconOk := shell_notifyIcon( cmd, @nid );
	if ((cmd = nim_add) or (cmd = nim_modify)) and (nid.hIcon <> 0) then destroyIcon( nid.hIcon );
end;
{ old code:
procedure tMainForm.trayIcon( cmd: dWord );
type
var
	nid: tNotifyIconData;
begin
	with nid do
		begin
		cbSize := sizeOf( nid );
		wnd := handle;
		uId := 0;
		if (cmd = nim_add) or (cmd = nim_modify) then
			begin
			uFlags := nif_icon or nif_message or nif_tip;
			uCallbackMessage := wm_trayIcon;
			if errorsLogged then hIcon := loadIcon( hInstance, 'TRAYICON_ERROR' )
			else hIcon := loadIcon( hInstance, 'TRAYICON' );
			strLCopy( szTip, pChar( appFullName + #13 + pnEmuleExe ), pred( sizeOf( szTip ) ) );
//			strLCopy( szTip, pChar( appFullName + #13 + eMulePath ), pred( sizeOf( szTip ) ) );
			end;
		end;
	trayIconOk := shell_notifyIcon( cmd, @nid );
	if ((cmd = nim_add) or (cmd = nim_modify)) and (nid.hIcon <> 0) then destroyIcon( nid.hIcon );
end;
}{ <<< 13 may 07: 0.48a support (new folders): Show eMule config folder path instead of eMule executable folder path }

procedure tMainForm.FormDestroy(Sender: TObject);
begin
	trayIcon( nim_delete );

	if logFiles <> nil then logFiles.free;
end;

procedure tMainForm.timerTimer(Sender: TObject);
var
	i: integer;
	log: tLogFile;
	fileName, s: string;
	fileDate,
	utcFileDate,
	fd: tFileDate;
	section, key: string;
	ini: tIniFile;
	n: int64;
	v: double;
	err: integer;
	failed: boolean;
	km: tKnownMet;
	kmr: tKnownMetRecord;
	p: integer;
	newLogs: boolean;
	onlinesig: tFileStream;
	l: integer;
	onlinesigEnabled: boolean;
	dt, dtNow: tDateTime;
begin
	timer.enabled := false;
	timer.interval := timer_normal;

	try

		if not closed then
			begin
			{ if the tray icon fails to load (at windows startup), try again }
			if not trayIconOk then trayIcon( nim_add );
			if not trayIconOk then
				begin
				timer.interval := timer_retry;
				exit;
				end;
			end;

		logFiles.load;

		try

			{ on first use and/or when the runtime log is missing, start viewer to create default logs and let the user select knownfiles to log }
			if (logFiles.count = 0) or (logFiles.runTimeLog = nil) then
				begin
				aOpen.execute;
				exit;
				end;

			{ known.met has changed? }
			s := pnEmuleConfig + fnKnownMet;
//			s := eMulePath + dnEmuleConfig + fnKnownMet;
			fileDate := fileAge( s );
			utcFileDate := fileDate;
			adjustNTFSDaylightFileDate( utcFileDate, s );

			{ 07 may 07: Don't try to read known.met if eMule is about to save it >>> }
			{ probably useless, but just in case... }
			dt := incMinute( fileDateToDateTime( fileDate ), kmSaveInterval );
			dtNow := now;
			if (dtNow < dt) and (millisecondsBetween( dt, dtNow ) < delayAfterDateChange) then { from 10 min 58 to 11 min }
				begin
				timer.interval := delayAfterDateChange * 2; { wait 4 extra seconds }
				exit;
				end
			else if (dtNow >= dt) and (millisecondsBetween( dtNow, dt ) < delayAfterDateChange) then { to 11 min to 11 min 02 }
				begin
				timer.interval := delayAfterDateChange; { wait 2 extra seconds }
				exit;
				end;
			{ <<< 07 may 07: Don't try to read known.met if eMule is about to save it }

			if (utcFileDate > logFiles.knownMetDate) and (millisecondsBetween( now, fileDateToDateTime( fileDate ) ) > delayAfterDateChange) then
				begin
				knownMetChgd := true;
				logFiles.knownMetDate := utcFileDate;
				end;

			ini := tIniFile.create( pnEmuleConfig + fnPreferencesIni );
//			ini := tIniFile.create( eMulePath + dnEmuleConfig + fnPreferencesIni );
			try
				onlinesigEnabled := ini.readBool( isEmule, ikOnlineSig, false );
				finally ini.free;
				end;
			{ 28 apr 07: bug fix: don't warn about onlineSig if no active log need it >>> }
{ old code:
			if not onlinesigEnabled then errorMsg( errOnlineSigDisabled );
}			{ <<< 28 apr 07: bug fix: don't warn about onlineSig if no active log need it }

			fileName := '';
			failed := false;
			newLogs := false;
			ini := nil;
			km := nil;
			onlinesig := nil;

			for i := 0 to pred( logFiles.count ) do
				begin
				log := logFiles.log[ i ];
				if log.flags and lfEmptyLog <> 0 then newLogs := true;

				{ data has to be retrieved from known.met, statistics.ini, preferences.ini...? }
				case log.what of
					lwIniFile:
						begin
						s := log.iniFile;
						if sameText( log.iniSection, isStatistics) and (sameText( s, fnStatisticsIni ) or sameText( s, fnPreferencesIni )) then
							begin { for old eMule versions, when the stats were saved in preferences.ini }
							s := pnEmuleConfig + fnStatisticsIni;
//							s := eMulePath + dnEmuleConfig + fnStatisticsIni;
							if not fileExists( s ) then s := pnEmuleConfig + fnPreferencesIni;
//							if not fileExists( s ) then s := eMulePath + dnEmuleConfig + fnPreferencesIni;
							end
						else
							s := pnEmuleConfig + log.iniFile;
//							s := eMulePath + dnEmuleConfig + log.iniFile;
						end;
					lwQueuedClients:
						begin
						s := pnEmuleData + fnOnlineSigDat;
//						s := eMulePath + fnOnlineSigDat;
						{ skip this log if onlineSig is not enabled }
						if not fileExists( s ) or not onlinesigEnabled then
							{ 28 apr 07: bug fix: don't warn about onlineSig if no active log need it >>> }
							begin
							errorMsg( errOnlineSigDisabled );
							continue;
							end;
{ old code:
							continue;
}							{ <<< 28 apr 07: bug fix: don't warn about onlineSig if no active log need it }
						end;
					else
						s := pnEmuleConfig + fnKnownMet;
//						s := eMulePath + dnEmuleConfig + fnKnownMet;
					end;

				{ open the data file.
				the log file names start with "preferences.ini", "statistics.ini", "Queued clients" or an hex hash,
				and they are ordered alphabetically, so each data file will be opened/closed only once;
				... I don't know if some mods store interresting values in an ini file whose first letter is between A and F... }
				if s <> fileName then
					begin
					fileName := s;
					fileDate := fileAge( fileName );
					utcFileDate := fileDate;
					adjustNTFSDaylightFileDate( utcFileDate, fileName );
					{ close previous file }
					if ini <> nil then freeAndNil( ini );
					if onlinesig <> nil then freeAndNil( onlinesig );
					if km <> nil then freeAndNil( km );
					{ try to open new one }
					if (millisecondsBetween( now, fileDateToDateTime( fileDate ) ) <= delayAfterDateChange) then
					{ too soon, emule may still be writing the file }
						begin
						{ retry in 2 seconds }
						timer.interval := timer_retry;
						failed := true;
						end
					else
						begin
						try
							case log.what of
								lwIniFile:
									{ open ini file; this should not fail }
									ini := tIniFile.create( fileName );
								lwQueuedClients:
									{ open onlinesig.dat file; this may fail if emule is currently accessing it }
									onlinesig := tFileStream.create( fileName, fmOpenRead or fmShareDenyNone );
								else
									{ open known.met; this may fail if emule is currently accessing it }
									begin
									km := tKnownMet.create( fileName );
									try
										km.load;
										except
											km.free;
											raise;
										end;
									end;
								end;
							failed := false;
							except
								{ retry in 2 seconds }
								timer.interval := timer_retry;
								failed := true;
								ini := nil;
								onlinesig := nil;
								km := nil;
							end;
						end;
					end;

				{ Read requested data }
				if not failed and (utcFileDate > log.lastRec.date) then
					begin
					try
						case log.what of
							lwIniFile:
								begin
								section := log.iniSection;
								key := log.iniKey;
								s := ini.readString( section, key, '' );
								if s <> '' then
									begin
									try
										{$R+}
										case log.dataType of
											ltByte:
												begin
												n := strToInt64( s );
												if (n < -128) or (n > 127) then raise eConvertError.create( '' );
												pLogRecByte( log.lastRec )^.data := n;
												end;
											ltWord:
												begin
												n := strToInt64( s );
												if (n < -32768) or (n > 32767) then raise eConvertError.create( '' );
												pLogRecWord( log.lastRec )^.data := n;
												end;
											ltInteger:
												begin
												n := strToInt64( s );
												if (n < low( integer )) or (n > maxInt) then raise eConvertError.create( '' );
												{ 29 apr 07: bug fix: Support for ConnRunTime reset >>> }
												if log = logFiles.runtimeLog then
													begin
													if pLogRecInteger( log.lastRec )^.data > n then
														n := n + pLogRecInteger( log.lastRec )^.data;
													end;
												{ 29 apr 07: bug fix: Support for ConnRunTime reset >>> }
												pLogRecInteger( log.lastRec )^.data := n;
												end;
											ltInt64:
												pLogRecInt64( log.lastRec )^.data := strToInt64( s );
											else
												begin
												val( s, v, err );
												if err = 0 then pLogRecReal( log.lastRec )^.data := v
												else
													raise eConvertError.create( '' );
												end;
											end;
										log.lastRec^.date := utcFileDate; { skipped if a conversion error occurs }
										except
											errorMsg( format( errLogIniUnexpectedType, [log.name] ) );
										end;
									end
								else
									errorMsg( format( errLogIniNoSuchValue, [log.name] ) );
								end;
							lwQueuedClients:
								begin
								try
									l := onlinesig.size;
									setLength( s, l );
									onlinesig.read( s[ 1 ], l );
									fd := fileAge( fileName );
									adjustNTFSDaylightFileDate( fd, fileName );
									if fd = utcFileDate then
										begin
										l := lastDelimiter( '|', s );
										if l > 0 then
											begin
											n := strToInt( copy( s, succ( l ), maxInt ) );
											pLogRecInteger( log.lastRec )^.data := n;
											log.lastRec^.date := utcFileDate;
											end
										else
											errorMsg( errInvalidOnlineSig );
										end
									else
										{file has been modified while reading -> retry in 2 seconds }
										timer.interval := timer_retry;
									except
										on e: exception do errorMsg( format( errLoadOnlineSig, [e.message] ) );
									end;
								end;
							else { lwKnownMet: }
								begin
								if km.list.findByHash( log.name, p ) then
									begin
									kmr := km.list.records[ p ];
									{ 07 may 07: Optimizations >>> }
									with pLogRecKnownMet( log.lastRec )^.data do
										begin
										requested := kmr.requested;
										accepted := kmr.accepted;
										transferred := kmr.transferred;
										end;
{ old code:
									pLogRecKnownMet( log.lastRec )^.data.requested := kmr.requested;
									pLogRecKnownMet( log.lastRec )^.data.accepted := kmr.accepted;
									pLogRecKnownMet( log.lastRec )^.data.transferred := kmr.transferred;
}									{ <<< 07 may 07: Optimizations }
									log.lastRec^.date := utcFileDate;
									end
								else
									errorMsg( format( errLogKFNoSuchHash, [log.name] ) );
								end;
							end; { case log.what of }
						except on e: exception do
							errorMsg( format( errLogCantReadData, [log.name, e.message] ) );
						end;
					end;
				end; { for i := 0 to pred( logFiles.count ) do }

			{ close data file }
			if ini <> nil then ini.free;
			if onlinesig <> nil then onlinesig.free;
			if km <> nil then freeAndNil( km );

			{ append new values to the logs }
			if (knownMetChgd or closed or newLogs) and (timer.interval = timer_normal) then
				begin
				for i := 0 to pred( logFiles.count ) do
					begin
					{ 07 may 07: Optimizations >>> }
					with logFiles.log[ i ] do
						begin
						{ skip Queued clients log if onlineSig is not enabled }
						if (what = lwQueuedClients) and (not fileExists( pnEmuleData + fnOnlineSigDat ) or not onlinesigEnabled) then
//						if (what = lwQueuedClients) and (not fileExists( eMulePath + fnOnlineSigDat ) or not onlinesigEnabled) then
							continue;
						if knownMetChgd or closed or (flags and lfEmptyLog <> 0) then
							append;
						end;
(* old code:
					log := logFiles.log[ i ];
					{ skip Queued clients log if onlineSig is not enabled }
					if (log.what = lwQueuedClients) and (not fileExists( eMulePath + fnOnlineSigDat ) or not onlinesigEnabled) then
						continue;
					if knownMetChgd or closed or (log.flags and lfEmptyLog <> 0) then
						log.append;
*)					{ <<< 07 may 07: Optimizations }
					end;
				knownMetChgd := false;
				end;

			finally
				logFiles.unload;
			end;
		finally
			{ 07 may 07: Bug fix: possible infinite loop on log updates -> timeout = 15 failed attempts >>> }
			if timer.interval = timer_normal then
				failures := 0
			else
				begin
				inc( failures );
				if failures >= 15 then
					begin
					application.messageBox( pChar( format( msgFatalError, [failures] ) ), pChar( appFullName ), mb_Ok or mb_iconError );
					application.terminate;
					end;
				end;
			if closed and (timer.interval = timer_normal) then
			{ <<< 07 may 07: Bug fix: possible infinite loop on log updates -> timeout = 15 failed attempts }
				application.terminate
			else
				begin
				timerStart := getTickCount;
				timer.enabled := true;
				end;
		end;
end;

procedure tMainForm.wmTrayIcon( var message: tMessage );
var
	pt: tPoint;
begin
	if message.lParam = wm_lButtonDblClk then
		aOpen.execute
	else if message.lParam = wm_rButtonDown then
		begin
		pt := mouse.cursorPos;
		trayPopup.Popup( pt.x, pt.y );
		end;
	inherited;
end;

procedure tMainForm.aOpenExecute(Sender: TObject);
//var
//	path: string;
begin
	execCommand( pnEFSExe + fnViewer, '', sw_showNormal );
//	path := extractFilePath( application.exeName ) + fnViewer;
//	execCommand( path, '', sw_showNormal );
end;

procedure tMainForm.aExitExecute(Sender: TObject);
var
	dw: dWord;
begin
	{ before terminating, update logs if necessary }
	closed := true;
	timer.interval := timer_init; { as soon as possible }

	{ close viewer }
	dw := bsm_applications;
	broadcastSystemMessage( bsf_ignoreCurrentTask, @dw, closeMsg, 0, 0 );
end;

procedure tMainForm.aOpenUpdate(Sender: TObject);
var
	h: tHandle;
begin
	{ 07 may 07: Disable menus when closing app (while waiting for logs update) >>> }
	if closed then
		aOpen.enabled := false
	else
		begin
	{ <<< 07 may 07: Disable menus when closing app (while waiting for logs update) }
		h := createMutex( nil, false, pChar( mutexPath + mnViewer ) );
		aOpen.enabled := (getLastError <> error_already_exists) and (getLastError <> error_access_denied);
		if h <> 0 then closeHandle( h );
	{ 07 may 07: Disable menus when closing app (while waiting for logs update) >>> }
		end;
	{ <<< 07 may 07: Disable menus when closing app (while waiting for logs update) }
end;

procedure tMainForm.wndProc( var message: tMessage );
var
	dw: dWord;
	l: int64;
begin
	if message.msg = taskBarCreatedMsg then
		begin
		trayIconOk := false;
		trayIcon( nim_add );
		end
	else if message.msg = suspendMsg then
		begin
		if not suspended or timer.enabled then
			begin
			timer.enabled := false;
			suspended := true;
			end;
		dw := bsm_applications;
		{ 29 apr 07: bug fix: Support for ConnRunTime reset >>> }
		broadcastSystemMessage( bsf_ignoreCurrentTask, @dw, suspendedMsg, message.wParam, 0 );
{ old code:
		broadcastSystemMessage( bsf_ignoreCurrentTask, @dw, suspendedMsg, 0, 0 );
}		{ <<< 29 apr 07: bug fix: Support for ConnRunTime reset }
		end
	else if message.msg = resumeMsg then
		begin
		if suspended then
			begin
			if timer.interval > timer_retry then
				begin
				dw := getTickCount;
				l := timer_normal - (dw - timerStart);
				if l < 0 then l := 1;
				timer.interval := l;
				end;
			suspended := false;
			end;
		timer.enabled := true;
		dw := bsm_applications;
		{ 29 apr 07: bug fix: Support for ConnRunTime reset >>> }
		broadcastSystemMessage( bsf_ignoreCurrentTask, @dw, resumedMsg, message.wParam, 0 );
{ old code:
		broadcastSystemMessage( bsf_ignoreCurrentTask, @dw, resumedMsg, 0, 0 );
}		{ <<< 29 apr 07: bug fix: Support for ConnRunTime reset }
		end
	else if message.msg = newLogsMsg then
		begin
		timer.enabled := false;
		timer.interval := timer_retry;
		timer.enabled := true;
		end
	else if message.msg = errorsViewedMsg then
		begin
		errorsLogged := false;
		trayIcon( nim_modify );
		end;

	inherited wndProc( message );
end;

{ 07 may 07: Disable menus when closing app (while waiting for logs update) >>> }
procedure tMainForm.aExitUpdate(Sender: TObject);
begin
	aExit.enabled := not closed;
end;
{ <<< 07 may 07: Disable menus when closing app (while waiting for logs update) }

end.
