{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit colorUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ActnList, Menus, ExtCtrls, Series, TeEngine, TeeProcs,
  Chart;

type
  tColorForm = class(TForm)
    colorDialog: TColorDialog;
	seriesBox: TListBox;
    PopupMenu1: TPopupMenu;
    mEditcolor: TMenuItem;
    ActionList1: TActionList;
    aEditColor: TAction;
    topPanel: TPanel;
    bottomPanel: TPanel;
    okBtn: TButton;
    cancelBtn: TButton;
    Label1: TLabel;
    reqChart: TChart;
    Series21: TBarSeries;
    PointSeries1: TLineSeries;
    Series2: TLineSeries;
    transChart: TChart;
    BarSeries1: TBarSeries;
    LineSeries1: TLineSeries;
    lumBar: TScrollBar;
    Label2: TLabel;
	procedure seriesBoxDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
	procedure seriesBoxDblClick(Sender: TObject);
	procedure aEditColorExecute(Sender: TObject);
	procedure aEditColorUpdate(Sender: TObject);
	procedure seriesBoxClick(Sender: TObject);
	procedure FormCreate(Sender: TObject);
    procedure lumBarChange(Sender: TObject);
  private
  public
  end;

var
	colorForm: tColorForm;
	deltaLum: double = 0.15;

procedure colorToHSL( color: tColor; var h, s, l: double ); { RGB to Hue/Saturation/Luminosity }
function HSLToColor( h, s, l: double ): tColor; { Hue/Saturation/Luminosity to RGB }
{ Note: In the Color dialog box, the saturation and luminosity values must be in the range 0 through 240,
and the hue value must be in the range 0 through 239.
Here, these are floating point values between 0 and 1.
Algorithms found at: http://www.easyrgb.com/math.php }
                                                                
procedure getRequestsColors( color: tColor; var rColor, aColor: tColor; lumOffset: double );
{ Returns the colors for the requests and accepted graphs, with a +/-deltaLum luminosity variation from the given color;
if color is too dark or too light, one of the returned colors might be black or white... }

implementation

{$R *.dfm}

procedure colorToHSL( color: tColor; var h, s, l: double );
var
	c: integer;
	r, g, b: double;
	min, max: double;
	delMax, delR, delG, delB: double;
begin
	c := colorToRGB( color );
	r := (c and $FF) / 255;
	g := ((c and $FF00) shR 8) / 255;
	b := ((c and $FF0000) shR 16) / 255;

	if (r <= g) and (r <= b) then min := r
	else if (g <= r) and (g <= b) then min := g
	else min := b;

	if (r >= g) and (r >= b) then max := r
	else if (g >= r) and (g >= b) then max := g
	else max := b;

	delMax := max - min;

	l := (max + min) / 2;

	if delMax = 0 then
		begin
		h := 0;
		s := 0;
		end
	else
		begin
		if l < 0.5 then s := delMax / (max + min)
		else s := delMax / (2 - max - min);

		delR := (((max - r) / 6) + (delMax / 2)) / delMax;
		delG := (((max - g) / 6) + (delMax / 2)) / delMax;
		delB := (((max - b) / 6) + (delMax / 2)) / delMax;

		if (r = max ) then h := delB - delG
		else if (g = max ) then h := (1 / 3) + delR - delB
		else if (b = max ) then h := (2 / 3) + delG - delR;

		if (h < 0) then h := h + 1;
		if (H > 1) then h := h - 1;
		end;
end;

function HSLToColor( h, s, l: double ): tColor;

	function hueToRGB( v1, v2, vh: double ): double;
	begin
		if (vh < 0) then vh := vh + 1;
		if (vh > 1) then vh := vh - 1;
		if ((6 * vh) < 1) then result := (v1 + (v2 - v1) * 6 * vh)
		else if ((2 * vh) < 1) then result := v2
		else if ((3 * vh) < 2) then result := (v1 + (v2 - v1) * ((2 / 3) - vh) * 6)
		else result := v1;
	end;

var
	r, g, b: integer;
	v1, v2: double;
begin
	if (s = 0) then
		begin
		r := round( l * 255 );
		g := round( l * 255 );
		b := round( l * 255 );
		end
	else
		begin
		if (l < 0.5) then v2 := l * (1 + s)
		else v2 := (l + s) - (s * l);

		v1 := 2 * l - v2;

		r := round( 255 * hueToRGB( v1, v2, h + (1 / 3) ) );
		g := round( 255 * hueToRGB( v1, v2, h ) );
		b := round( 255 * hueToRGB( v1, v2, h - (1 / 3) ) );
		end;
	result := r or (g shL 8) or (b shL 16);
end;

procedure getRequestsColors( color: tColor; var rColor, aColor: tColor; lumOffset: double );
var
	h, s, l: double;
begin
	colorToHSL( color, h, s, l );
	if l > 1 - lumOffset then l := 1 - 2 * lumOffset else if l > lumOffset then l := l - lumOffset else l := 0;
	rColor := HSLToColor( h, s, l );
	l := l + 2 * lumOffset;
	aColor := HSLToColor( h, s, l );
end;

procedure tColorForm.seriesBoxDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
	with seriesBox.canvas do
		begin
		fillRect( rect );
		font.color := tColor( seriesBox.items.objects[ index ] );
		textOut( rect.left + 2, rect.top + 1, seriesBox.items[ index ] );
		end;
end;

procedure tColorForm.seriesBoxDblClick(Sender: TObject);
begin
	if seriesBox.itemIndex >= 0 then
		begin
		colorDialog.color := tColor( seriesBox.items.objects[ seriesBox.itemIndex ] );
		if colorDialog.execute then
			begin
			seriesBox.items.objects[ seriesBox.itemIndex ] := tObject( colorDialog.color );
			seriesBox.invalidate;
			seriesBoxClick( seriesBox );
			end;
		end;
end;

procedure tColorForm.aEditColorExecute(Sender: TObject);
begin
	seriesBoxDblClick( seriesBox );
end;

procedure tColorForm.aEditColorUpdate(Sender: TObject);
begin
	aEditColor.enabled := (seriesBox.itemIndex >= 0);
end;

procedure tColorForm.seriesBoxClick(Sender: TObject);
var
	index: integer;
	col, rCol, aCol: tColor;
begin
	reqChart.series[ 0 ].seriesColor := clWindow;
	reqChart.series[ 1 ].seriesColor := clWindow;
	reqChart.series[ 2 ].seriesColor := clWindow;
	transChart.series[ 0 ].seriesColor := clWindow;
	transChart.series[ 1 ].seriesColor := clWindow;
	index := seriesBox.itemIndex;
	if index >= 0 then
		begin
		col := tColor( seriesBox.items.objects[ index ] );
		if index = 0 then
			begin
			reqChart.series[ 0 ].seriesColor := col;
			transChart.series[ 0 ].seriesColor := col;
			end
		else if index <= 3 then
			begin
			getRequestsColors( col, rCol, aCol, lumBar.position / 100 );
			reqChart.series[ 1 ].seriesColor := rCol;
			reqChart.series[ 2 ].seriesColor := aCol;
			transChart.series[ 1 ].seriesColor := col;
			end
		else
			transChart.series[ 1 ].seriesColor := col;
		end;
end;

procedure tColorForm.FormCreate(Sender: TObject);
begin
	reqChart.series[ 0 ].AddXY( 1, 1 );
	reqChart.series[ 0 ].AddXY( 2, 1 );
	reqChart.series[ 0 ].AddXY( 4, 1 );

	reqChart.series[ 1 ].AddXY( 0, 1 );
	reqChart.series[ 1 ].AddXY( 1, 2 );
	reqChart.series[ 1 ].AddXY( 2, 5 );
	reqChart.series[ 1 ].AddXY( 3, 3 );
	reqChart.series[ 1 ].AddXY( 4, 4 );
	reqChart.series[ 1 ].AddXY( 5, 2 );

	reqChart.series[ 2 ].AddXY( 0, 1 );
	reqChart.series[ 2 ].AddXY( 1, 1.5 );
	reqChart.series[ 2 ].AddXY( 2, 4 );
	reqChart.series[ 2 ].AddXY( 3, 2 );
	reqChart.series[ 2 ].AddXY( 4, 4 );
	reqChart.series[ 2 ].AddXY( 5, 1 );

	transChart.series[ 0 ].AddXY( 1, 1 );
	transChart.series[ 0 ].AddXY( 2, 1 );
	transChart.series[ 0 ].AddXY( 4, 1 );

	transChart.series[ 1 ].AddXY( 0, 1 );
	transChart.series[ 1 ].AddXY( 1, 2 );
	transChart.series[ 1 ].AddXY( 2, 5 );
	transChart.series[ 1 ].AddXY( 3, 3 );
	transChart.series[ 1 ].AddXY( 4, 4 );
	transChart.series[ 1 ].AddXY( 5, 2 );
end;

procedure tColorForm.lumBarChange(Sender: TObject);
begin
	seriesBoxClick( seriesBox );
end;

end.
