{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit combineUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, math, comCtrls,
  basicDefsUnit, AppEvnts;

type
  tCombineForm = class(TForm)
	topPanel: TPanel;
	bottomPanel: TPanel;
	okBtn: TButton;
	cancelButton: TButton;
	opBox: TRadioGroup;
	Label1: TLabel;
	titleBox: TEdit;
	flagsShowWithRequestsBox: TCheckBox;
	Label2: TLabel;
	multiplierBox: TEdit;
	Label3: TLabel;
	unitsBox: TEdit;
	AppEvents: TApplicationEvents;
    useMaxValueBox: TCheckBox;
    maxValueBox: TEdit;
	procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
	procedure updateOkBtn(Sender: TObject);
	procedure FormShow(Sender: TObject);
	procedure FormCreate(Sender: TObject);
	procedure AppEventsShowHint(var HintStr: String; var CanShow: Boolean; var HintInfo: THintInfo);
	procedure FormDestroy(Sender: TObject);
    procedure useMaxValueBoxClick(Sender: TObject);
  private
	fLogs: tCombinedLogs;
	fLogFile: tLogFile;
	hintDelay: integer;
	procedure setLogs( index: integer; log: tLogFile );
	procedure setLogFile( aLogFile: tLogFile );
	function getFileName: string;
	procedure setFileName( const aFileName: string );
	function validate: boolean;
	function getMultiplier: double;
	procedure setMultiplier( aMultiplier: double );
	function getFlags: byte;
	procedure setFlags( aFlags: byte );
	function getUnits: string;
	procedure setUnits( const aUnits: string );
	function getOperation: tCombineOp;
	procedure setOperation( aOperation: tCombineOp );
	function getLog1: shortString;
	function getLog2: shortString;
	function getMaxValue: double;
	procedure setMaxValue( aMaxValue: double );
  public
	property logs[ index: integer ]: tLogFile write setLogs; { create new }
	property logFile: tLogFile read fLogFile write setLogFile; { edit existing }
	property fileName: string read getFileName write setFileName;
	property multiplier: double read getMultiplier write setMultiplier;
	property flags: byte read getFlags write setFlags;
	property units: string read getUnits write setUnits;
	property operation: tCombineOp read getOperation write setOperation;
	property log1: shortString read getLog1;
	property log2: shortString read getLog2;
	property maxValue: double read getMaxValue write setMaxValue;
  end;

var
  combineForm: tCombineForm;

implementation

{$R *.dfm}

function tCombineForm.validate: boolean;
var
	ln: string;
	v: double;
begin
	if maxValueBox.text = '' then
		v := nan
	else
		try
			v := strToFloat( maxValueBox.text );
			except v := nan;
			end;
	ln := format( combinedLogFmt, [titleBox.text] );
	result := not isNAN( multiplier )
		and (titleBox.text <> '') and (lastDelimiter( '/\:*?"<>|', titleBox.text ) = 0)
		and (
			(fLogFile <> nil) and sameText( ln, fLogFile.name )
		or
			not fileExists( pnEFSLogs + ln + enLogExt )
			and not fileExists( pnEFSLogs + ln + disLogExt )
			and not fileExists( pnEFSLogs + ln + permLogExt )
(*			not fileExists( logsPath + ln + enLogExt )
			and not fileExists( logsPath + ln + disLogExt )
			and not fileExists( logsPath + ln + permLogExt )
*)		)
		and (not useMaxValueBox.checked or not isNAN( v ));
end;

function tCombineForm.getMultiplier: double;
var
	s: string;
	v: double;
	op: (opEq, opMul, opDiv);

	procedure doIt( p1, p2: integer );
	var
		p: integer;
		s1: string;
		v1: double;
	begin
		if (p1 > 0) and ((p1 < p2) or (p2 = 0)) then p := p1
		else if (p2 > 0) then p := p2
		else p := succ( length( s ) );
		s1 := trim( copy( s, 1, pred( p ) ) );
		s := trim( copy( s, succ( p ), maxInt ) );
		if (s1 = '') or ((p1 > 0) or (p2 > 0)) and (s = '') then
			v := nan
		else
			begin
			try
				v1 := strToFloat( s1 );
				case op of
					opEq:	v := v1;
					opMul:	v := v * v1;
					else	if v1 = 0 then v := nan
							else v := v / v1;
					end;
				except v := nan;
				end;
			if p = p1 then op := opMul
			else op := opDiv;
			end;
	end;

begin
	s := trim( multiplierBox.text );
	if s = '' then
		result := nan
	else
		begin
		op := opEq;
		v := 1;
		while (s <> '') and not isNAN( v ) do doIt( pos( '*', s ), pos( '/', s ) );
		if isNAN( v ) or (v = 0) then result := nan
		else result := v;
		end;
end;

procedure tCombineForm.setMultiplier( aMultiplier: double );
begin
	if aMultiplier = 1 then
		multiplierBox.text := '1/1'
	else
		multiplierBox.text := format( '%g', [aMultiplier] );
end;

procedure tCombineForm.setLogs( index: integer; log: tLogFile );
begin
	fLogs[ index ] := log;
	opBox.items.text := stringReplace( opBox.items.text, '%' + intToStr( succ( index ) ), log.iniCaption, [rfReplaceAll] );
end;

procedure tCombineForm.setLogFile( aLogFile: tLogFile );
begin
	fLogFile := aLogFile;
	opBox.items.text := stringReplace( stringReplace( opBox.items.text, '%1', iniLogNameToCaption( aLogFile.logNames[ 0 ] ), [rfReplaceAll] ), '%2', iniLogNameToCaption( aLogFile.logNames[ 1 ] ), [rfReplaceAll] );
end;

function tCombineForm.getFileName: string;
begin
	result := format( combinedLogFmt, [titleBox.text] );
end;

procedure tCombineForm.setFileName( const aFileName: string );
begin
	titleBox.text := copy( aFileName, succ( length( format( combinedLogFmt, [''] ) ) ), maxInt );
end;

procedure tCombineForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
	canClose := (modalResult = mrCancel) or validate;
end;

procedure tCombineForm.updateOkBtn(Sender: TObject);
begin
	okBtn.enabled := validate;
end;

procedure tCombineForm.FormShow(Sender: TObject);
begin
	okBtn.enabled := validate;
end;

function tCombineForm.getFlags: byte;
begin
	if flagsShowWithRequestsBox.checked then result := lfShowWithRequests
	else result := 0;
end;

procedure tCombineForm.setFlags( aFlags: byte );
begin
	flagsShowWithRequestsBox.checked := (aFlags and lfShowWithRequests <> 0);
end;

function tCombineForm.getUnits: string;
begin
	result := unitsBox.text;
end;

procedure tCombineForm.setUnits( const aUnits: string );
begin
	unitsBox.text := aUnits;
end;

function tCombineForm.getOperation: tCombineOp;
begin
	result := tCombineOp( opBox.itemIndex );
end;

procedure tCombineForm.setOperation( aOperation: tCombineOp );
begin
	opBox.itemIndex := ord( aOperation );
end;

function tCombineForm.getLog1: shortString;
begin
	result := fLogs[ 0 ].name;
end;

function tCombineForm.getLog2: shortString;
begin
	result := fLogs[ 1 ].name;
end;

procedure tCombineForm.FormCreate(Sender: TObject);
begin
	fLogFile := nil;
	hintDelay := application.hintHidePause;
	application.hintHidePause := 10000;
end;

procedure tCombineForm.AppEventsShowHint(var HintStr: String; var CanShow: Boolean; var HintInfo: THintInfo);
begin
	hintStr := stringReplace( stringReplace( stringReplace( stringReplace( hintStr, '\\', #1, [rfReplaceAll] ), '\n', #13#10, [rfReplaceAll] ), #1, '\', [rfReplaceAll] ), ';   ', #13#10, [rfReplaceAll] );
end;

procedure tCombineForm.FormDestroy(Sender: TObject);
begin
	application.hintHidePause := hintDelay;
end;

procedure tCombineForm.useMaxValueBoxClick(Sender: TObject);
begin
	if useMaxValueBox.checked then
		begin
		maxValueBox.enabled := true;
		if visible then maxValueBox.setFocus;
		end
	else
		maxValueBox.enabled := false;
end;

function tCombineForm.getMaxValue: double;
begin
	if useMaxValueBox.checked then result := strToFloat( maxValueBox.text )
	else result := nan;
end;

procedure tCombineForm.setMaxValue( aMaxValue: double );
begin
	if isNAN( aMaxValue ) then
		maxValueBox.enabled := false
	else
		begin
		useMaxValueBox.checked := true;
		maxValueBox.text := format( '%g', [aMaxValue] );
		end;
end;

end.
