{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit eventsLogEditUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, dateUtils, Menus, ActnList, comCtrls,
  basicDefsUnit, knownMetUnit;

resourceString
	errInvalidEventsLogEntry = 'Invalid value in %s at line %d';
	errSaveEventsLog = 'An error occurred while saving %s: %s';

const
	fnEventsLog = 'events.log';

type
	tEventsLog = class( tStringList )
		protected
			function compareStrings( const s1, s2: string ): integer; override;
			function getDates( index: integer ): tDateTime;
			function getEvents( index: integer ): string;
			function getHashes( index: integer ): string;
		public
			constructor create;
			procedure load;
			procedure save;
			property dates[ index: integer ]: tDateTime read getDates;
			property events[ index: integer ]: string read getEvents;
			property hashes[ index: integer ]: string read getHashes;
		end;

  tEventsLogEditForm = class(TForm)
	Panel1: TPanel;
	Panel2: TPanel;
	okBtn: TButton;
	cancelBtn: TButton;
	dateTimeBox: TEdit;
	Label1: TLabel;
	eventBox: TEdit;
	Label2: TLabel;
	eventsBox: TListBox;
	Label3: TLabel;
	listMenu: TPopupMenu;
	actions: TActionList;
    aAdd: TAction;
    Addnewevent1: TMenuItem;
	aDel: TAction;
    errorBox: TStaticText;
    fileBox: TComboBox;
    Label4: TLabel;
	procedure eventsBoxDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
	procedure eventsBoxClick(Sender: TObject);
	procedure textBoxChange(Sender: TObject);
	procedure FormCreate(Sender: TObject);
	procedure FormDestroy(Sender: TObject);
	procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
	procedure aAddExecute(Sender: TObject);
	procedure FormShow(Sender: TObject);
	procedure aDelExecute(Sender: TObject);
	procedure aDelUpdate(Sender: TObject);
	procedure eventsBoxEnter(Sender: TObject);
	procedure eventsBoxExit(Sender: TObject);
    procedure fileBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  protected
	fLog, fTempLog: tEventsLog;
	fDelShcut: tShortcut;
  private
  public
	property log: tEventsLog read fLog;
  end;

procedure removeFileEvents( const aHash: string );

var
  eventsLogEditForm: tEventsLogEditForm;

implementation

uses
	mainUnit;

{$R *.dfm}

var
	inEventsBoxClick: boolean = false;

constructor tEventsLog.create;
begin
	inherited;
	sorted := true;
	caseSensitive := false;
	duplicates := dupAccept;
end;

function tEventsLog.compareStrings( const s1, s2: string ): integer;
var
	p1, p2, p3, p4: integer;
	dt1, dt2: tDateTime;
	s3, s4: string;
	l3, l4: integer;
begin
	p1 := pos( '=', s1 );
	if p1 = 0 then p1 := succ( length( s1 ) );
	dt1 := strToDateTime( copy( s1, 1, pred( p1 ) ) );
	p2 := pos( '=', s2 );
	if p2 = 0 then p2 := succ( length( s2 ) );
	dt2 := strToDateTime( copy( s2, 1, pred( p2 ) ) );
	result := compareDateTime( dt1, dt2 );
	if result = 0 then
		begin
		s3 := copy( s1, succ( p1 ), maxInt );
		l3 := length( s3 );
		if (l3 >= 33) and (s3[ l3 - 32 ] = '|') then p3 := l3 - 32 else p3 := 0;
		s4 := copy( s2, succ( p2 ), maxInt );
		l4 := length( s4 );
		if (l4 >= 33) and (s4[ l4 - 32 ] = '|') then p4 := l4 - 32 else p4 := 0;
		if (p3 > 0) and (p4 = 0) then
			result := 1
		else if (p3 = 0) and (p4 > 0) then
			result := -1
		else if (p3 = 0) and (p4 = 0) then
			result := ansiCompareText( s3, s4 )
		else
			begin
			result := ansiCompareText( copy( s3, succ( p3 ), maxInt ), copy( s4, succ( p4 ), maxInt ) );
			if result = 0 then
				result := ansiCompareText( copy( s3, 1, pred( p3 ) ), copy( s4, 1, pred( p4 ) ) );
			end;
		end;
end;

function tEventsLog.getDates( index: integer ): tDateTime;
begin
	result := strToDateTime( names[ index ] );
end;

function tEventsLog.getEvents( index: integer ): string;
var
	l, i: integer;
begin
	result := strings[ index ];
	l := pos( '=', result );
	if l = 0 then
		result := ''
	else
		begin
		result := copy( result, succ( l ), maxInt );
		l := length( result );
		if (l >= 33) and (result[ l - 32] = '|') then
			begin
			for i := l - 31 to l do if not (result[ i ] in ['0'..'9', 'a'..'f', 'A'..'F']) then
				exit;
			setLength( result, l - 33 );
			end;
		end;

end;

function tEventsLog.getHashes( index: integer ): string;
var
	l, i: integer;
begin
	result := strings[ index ];
	l := pos( '=', result );
	if l = 0 then
		result := ''
	else
		begin
		result := copy( result, succ( l ), maxInt );
		l := length( result );
		if (l >= 33) and (result[ l - 32] = '|') then
			begin
			for i := l - 31 to l do if not (result[ i ] in ['0'..'9', 'a'..'f', 'A'..'F']) then
				begin
				result := '';
				exit;
				end;
			result := copy( result, l - 31, 32 );
			end
		else
			result := '';
		end;

end;

procedure tEventsLog.load;
var
	i: integer;
	fs: tFormatSettings;
	dt: tDateTime;
	s: string;
	p: integer;
begin
	clear;
	if fileExists( pnEFSConfig + fnEventsLog ) then
//	if fileExists( appPath + fnEventsLog ) then
		begin
		sorted := false;
		try
			loadFromFile( pnEFSConfig + fnEventsLog );
//			loadFromFile( appPath + fnEventsLog );
			getLocaleFormatSettings( $409, fs );
			for i := pred( count ) downTo 0 do
				begin
				try
					s := strings[ i ];
					p := pos( '=', s );
					if p = 0 then p := succ( length( s ) );
					dt := strToDateTime( trim( copy( s, 1, pred( p ) ) ), fs );
					strings[ i ] := dateTimeToStr( dt ) + '=' + trim( copy( s, succ( p ), maxInt ) );
					except
						delete( i );
						errorMsg( format( errInvalidEventsLogEntry, [fnEventsLog, succ( i )] ) );
					end;
				end;
			finally sorted := true;
			end;
		end;
end;

procedure tEventsLog.save;
var
	i, p: integer;
	fs: tFormatSettings;
	dt: tDateTime;
	f: tFileStream;
	s, v: string;
begin
	try
		f := tFileStream.create( pnEFSConfig + fnEventsLog, fmCreate );
//		f := tFileStream.create( appPath + fnEventsLog, fmCreate );
		try
			getLocaleFormatSettings( $409, fs );
			for i := 0 to pred( count ) do
				begin
				dt := strToDateTime( names[ i ] ); { should be already validated }
				v := strings[ i ];
				p := pos( '=', v );
				if p <> 0 then
					begin
					v := trim( copy( v, succ( p ), maxInt ) );
					if v <> '' then
						begin
						{ save the date/time in english (us) notation, so that it's independent from locale settings, but the file remains readable (not binary data...) }
						s := dateTimeToStr( dt, fs ) + '=' + v + #13#10;
						f.write( s[ 1 ], length( s ) );
						end;
					end;
				end;
			finally f.free;
			end;
		except
			on e: exception do errorMsg( format( errSaveEventsLog, [fnEventsLog, e.message] ) );
		end;
end;

procedure tEventsLogEditForm.eventsBoxDrawItem( control: tWinControl; index: integer; rect: tRect; state: tOwnerDrawState );
var
	hash: string;
	kf: tKnownMetRecord;
begin
	with (control as tListBox).canvas do
		begin
		fillRect( rect );
		textOut( rect.left + 2, rect.top + 1, fLog.names[ index ] );
		hash := fLog.hashes[ index ];
		kf := mainForm.displayedFile[ hash ];
		if (hash <> '') and (kf <> nil) then font.color := kf.color;
		textOut( rect.left + 2 + 150, rect.top + 1, fLog.events[ index ] );
		end;
end;

procedure tEventsLogEditForm.eventsBoxClick(Sender: TObject);
var
	n: integer;
	h: string;
begin
	inEventsBoxClick := true;
	if eventsBox.itemIndex >= 0 then
		begin
		dateTimeBox.text := eventsBox.items.names[ eventsBox.itemIndex ];
		eventBox.text := fLog.events[ eventsBox.itemIndex ];

		h := fLog.hashes[ eventsBox.itemIndex ];
		if h = '' then
			n := 0
		else
			begin
			n := fileBox.items.count;
			repeat
				dec( n );
				if tKnownMetRecord( fileBox.items.objects[ n ] ).hash.compareStr( h ) = 0 then
					break;
			until n = 0;
			end;
		fileBox.itemIndex := n;
		end
	else
		begin
		dateTimeBox.text := '';
		eventBox.text := '';
		end;
	errorBox.visible := false;
	inEventsBoxClick := false;
end;

procedure tEventsLogEditForm.textBoxChange(Sender: TObject);
var
	dt: tDateTime;
	p: integer;
	dts: string;
	ok: boolean;
begin
	if inEventsBoxClick then exit;

	ok := true;
	dt := 0;
	try
		dt := strToDateTime( dateTimeBox.text );
		except
			ok := false;
		end;

	if eventsBox.itemIndex >= 0 then
		fLog.delete( eventsBox.itemIndex );

	if ok then
		dts := dateTimeToStr( dt );

	if ok then
		begin
		if fileBox.itemIndex = 0 then
			p := fLog.add( dts + '=' + eventBox.text )
		else
			p := fLog.add( dts + '=' + eventBox.text + '|' + tKnownMetRecord( fileBox.items.objects[ fileBox.itemIndex ] ).hash.hashStr );
		end
	else
		p := -1;

	eventsBox.items.assign( fLog );
	eventsBox.itemIndex := p;

	errorBox.visible := not ok;
end;

procedure tEventsLogEditForm.FormCreate(Sender: TObject);
var
	i: integer;
	hash: string;
	kf: tKnownMetRecord;
	item: tListItem;
begin
	fLog := tEventsLog.create;
	fLog.load;
	fTempLog := tEventsLog.create;
	for i := pred( fLog.count ) downTo 0 do
		begin
		hash := fLog.hashes[ i ];
		kf := mainForm.displayedFile[ hash ];
		if (hash <> '') and (kf = nil) then
			begin
			fTempLog.add( fLog[ i ] );
			fLog.delete( i );
			end;
		end;
	eventsBox.items.assign( fLog );
	fDelShcut := aDel.shortCut;
	aDel.shortCut := tShortcut( 0 );

	if refKF1 <> nil then
		begin
		kf := tKnownMetRecord( refKF1.data );
		fileBox.items.addObject( kf.fileName, kf );
		end;
	if refKF2 <> nil then
		begin
		kf := tKnownMetRecord( refKF2.data );
		fileBox.items.addObject( kf.fileName, kf );
		end;
	item := mainForm.knownFilesList.selected;
	while (item <> nil) do
		begin
		if (item <> refKF1) and (item <> refKF2) then
			begin
			kf := tKnownMetRecord( item.data );
			if kf.color <> clWindow then fileBox.items.addObject( kf.fileName, kf );
			end;
		item := mainForm.knownFilesList.getNextItem( item, sdAll, [isSelected] );
		end;
	fileBox.itemIndex := 0;
end;

procedure tEventsLogEditForm.FormDestroy(Sender: TObject);
begin
	fTempLog.free;
	fLog.free;
end;

procedure tEventsLogEditForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
	i: integer;
begin
	if canClose and (modalResult = mrOk) then
		begin
		for i := 0 to pred( fTempLog.count ) do fLog.add( fTempLog[ i ] );
		fLog.save;
		end;
end;

procedure tEventsLogEditForm.aAddExecute(Sender: TObject);
begin
	eventsBox.itemIndex := -1;
	dateTimeBox.text := '';
	eventBox.text := '';
	dateTimeBox.setFocus;
end;

procedure tEventsLogEditForm.FormShow(Sender: TObject);
var
	p: integer;
begin
	if dateTimeBox.text <> '' then
		begin
		fLog.find( dateTimeBox.text, p );
		if (p >= 0) and (p < fLog.count) and (fLog.names[ p ] = dateTimeBox.text) then
			begin
			eventsBox.itemIndex := p;
			eventsBoxClick( eventsBox );
			end;

		if eventBox.enabled then eventBox.setFocus else dateTimeBox.setFocus;
		end;
end;

procedure tEventsLogEditForm.aDelExecute(Sender: TObject);
var
	p: integer;
begin
	p := eventsBox.itemIndex;
	fLog.delete( p );
	eventsBox.items.assign( fLog );
	if p >= fLog.count then p := pred( fLog.count );
	eventsBox.itemIndex := p;
	eventsBoxClick( eventsBox );
end;

procedure tEventsLogEditForm.aDelUpdate(Sender: TObject);
begin
	aDel.enabled := (eventsBox.itemIndex >= 0);
end;

procedure tEventsLogEditForm.eventsBoxEnter(Sender: TObject);
begin
	aDel.shortCut := fDelShcut;
end;

procedure tEventsLogEditForm.eventsBoxExit(Sender: TObject);
begin
	aDel.shortCut := tShortcut( 0 );
end;

procedure tEventsLogEditForm.fileBoxDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
	kf: tKnownMetRecord;
begin
	with fileBox.canvas do
		begin
		fillRect( rect );
		kf := tKnownMetRecord( fileBox.items.objects[ index ] );
		if kf = nil then font.color := clWindowText
		else font.color := kf.color;
		textOut( rect.left + 2, rect.top + 1, fileBox.items[ index ] );
		end;
end;

procedure removeFileEvents( const aHash: string );
var
	evtLog: tEventsLog;
	i: integer;
	b: boolean;
	h: tHash;
	hash: string;
begin
	h := tHash.createStr( aHash );
	try
		evtLog := tEventsLog.create;
		try
			evtLog.load;
			b := false;
			for i := pred( evtLog.count ) downTo 0 do
				begin
				hash := evtLog.hashes[ i ];
				if (hash <> '') and (h.compareStr( hash ) = 0) then
					begin
					evtLog.delete( i );
					b := true;
					end;
				end;
			if b then evtLog.save;
			finally
				evtLog.free;
			end;
		finally
			h.free;
		end;
end;

end.
