{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit newLogWizardUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, iniFiles, math, StdCtrls, ExtCtrls, AppEvnts,
  basicDefsUnit;

resourceString
	rsSpecialLogs = '- Special logs -';
	rsFileNotFound = '[File not found]';
	rsOnlineSigDisabled = '[Online signature is currently disabled]';

type
  tNewLogWizard = class(TForm)
	tree: TTreeView;
	Label1: TLabel;
	typeBox: TRadioGroup;
	flagsBox: TGroupBox;
	flagsCumulBox: TCheckBox;
	multiplierBox: TEdit;
	Label2: TLabel;
	Label3: TLabel;
	unitsBox: TEdit;
	okBtn: TButton;
	cancelBtn: TButton;
	flagsSavedOnChangeBox: TCheckBox;
	appEvents: TApplicationEvents;
	flagsShowWithRequestsBox: TCheckBox;
	procedure treeChange(Sender: TObject; Node: TTreeNode);
	function getMultiplier: double;
	function validate: boolean;
	procedure multiplierBoxChange(Sender: TObject);
	procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
	procedure appEventsShowHint(var HintStr: String; var CanShow: Boolean; var HintInfo: THintInfo);
	procedure FormCreate(Sender: TObject);
	procedure FormDestroy(Sender: TObject);
  protected
	fLogFile: tLogFile;
	fQueuedClientsNode: tTreeNode;
	procedure setLogFile( aLogFile: tLogFile );
  public
	fileName: string;
	dataType: tLogType;
	flags: byte;
	units: string;
	hintDelay: integer;
	property logFile: tLogFile read fLogFile write setLogFile;
	property multiplier: double read getMultiplier;
  end;

var
  newLogWizard: tNewLogWizard;

implementation

{$R *.dfm}

procedure tNewLogWizard.setLogFile( aLogFile: tLogFile );
var
	fileNode,
	sectionNode,
	keyNode: tTreeNode;

	procedure addSpecialLogs;
	var
		onlinesig: tFileStream;
		l: integer;
		s: string;
		found: boolean;
		ini: tIniFile;
	begin
		if (aLogFile = nil) or (aLogFile.what = lwQueuedClients) then
			begin
			if fileNode = nil then fileNode := tree.items.addChild( nil, rsSpecialLogs );
			ini := tIniFile.create( pnEmuleConfig + fnPreferencesIni );
//			ini := tIniFile.create( eMulePath + dnEmuleConfig + fnPreferencesIni );
			try
				found := ini.readBool( isEmule, ikOnlineSig, false );
				finally ini.free;
				end;
			if not found then
				s := rsOnlineSigDisabled
			else
				begin
				found := fileExists( pnEmuleData + fnOnlineSigDat );
//				found := fileExists( eMulePath + fnOnlineSigDat );
				if found then
					begin
					try
						onlinesig := tFileStream.create( pnEmuleData + fnOnlineSigDat, fmOpenRead or fmShareDenyNone );
//						onlinesig := tFileStream.create( eMulePath + fnOnlineSigDat, fmOpenRead or fmShareDenyNone );
						try
							l := onlinesig.size;
							setLength( s, l );
							onlinesig.read( s[ 1 ], l );
							l := lastDelimiter( '|', s );
							if l > 0 then s := copy( s, succ( l ), maxInt ) else s := '';
							finally onlinesig.free;
							end;
						except s := '';
						end;
					end
				else
					s := rsFileNotFound;
				end;
			sectionNode := tree.items.addChild( fileNode, fnOnlineSigDat );
			keyNode := tree.items.addChild( sectionNode, fnQueueLog + ' = ' + s );
			fQueuedClientsNode := keyNode;
			end;
	end;

	procedure initNewLog;
	var
		sr: tSearchRec;
		err: integer;
		ini: tIniFile;
		sections,
		values: tStringList;
		i, j: integer;
		s: string;
		v: double;
		code: integer;
	begin
		err := findFirst( pnEmuleConfig + '*.ini', faArchive, sr );
//		err := findFirst( eMulePath + dnEmuleConfig + '*.ini', faArchive, sr );
		try
			while err = 0 do
				begin
				fileNode := nil;
				try
					ini := tIniFile.create( pnEmuleConfig + sr.name );
//					ini := tIniFile.create( eMulePath + dnEmuleConfig + sr.name );
					try
						sections := tStringList.create;
						try
							sections.sorted := true;
							sections.caseSensitive := false;
							sections.duplicates := dupIgnore;
							ini.readSections( sections );
							for i := 0 to pred( sections.count ) do
								begin
								sectionNode := nil;
								values := tStringList.create;
								try
									values.sorted := true;
									values.caseSensitive := false;
									values.duplicates := dupIgnore;
									ini.readSection( sections[ i ], values );
									for j := 0 to pred( values.count ) do
										begin
										s := ini.readString( sections[ i ], values[ j ], '' );
										if s <> '' then
											begin
											val( s, v, code );
											if v = 0 then ;
											if code = 0 then
												begin
												if fileNode = nil then fileNode := tree.items.addChild( nil, sr.name );
												if sectionNode = nil then sectionNode := tree.items.addChild( fileNode, sections[ i ] );
												tree.items.addChild( sectionNode, values[ j ] + ' = ' + s );
												end;
											end;
										end;
									finally values.free;
									end;
								end;
							finally sections.free;
							end;
						finally ini.free;
						end;
					except on e: exception do errorMsg( e.message );
					end;

				err := findNext( sr );
				end;
			finally findClose( sr );
			end;
		fileNode := nil;
		addSpecialLogs;
	end;

	procedure iniLogProps;
	var
		ini: tIniFile;
		s: string;
	begin
		if fLogFile.what = lwIniFile then
			begin
			try
				ini := tIniFile.create( pnEmuleConfig + fLogFile.iniFile );
//				ini := tIniFile.create( eMulePath + dnEmuleConfig + fLogFile.iniFile );
				try
					s := ini.readString( fLogFile.iniSection, fLogFile.iniKey, '' );
					finally ini.free;
					end;
				except s := '';
				end;
			fileNode := tree.items.addChild( nil, fLogFile.iniFile );
			sectionNode := tree.items.addChild( fileNode, fLogFile.iniSection );
			keyNode := tree.items.addChild( sectionNode, fLogFile.iniKey + ' = ' + s );
			end
		else { if fLogFile.what = lwQueuedClients then }
			begin
			fileNode := nil;
			addSpecialLogs;
			end;
		fileNode.expand( true );
		keyNode.selected := true;

		typeBox.enabled := false;
		flagsSavedOnChangeBox.enabled := false;

		typeBox.itemIndex := pred( ord( fLogFile.dataType ) );
		flagsCumulBox.checked := (fLogFile.flags and lfCumulativeValue <> 0);
		flagsCumulBox.enabled := (fLogFile.what = lwIniFile);
		flagsSavedOnChangeBox.checked := (fLogFile.flags and lfSavedOnChange <> 0);
		flagsShowWithRequestsBox.checked := (fLogFile.flags and lfShowWithRequests <> 0);
		multiplierBox.text := format( '%g', [fLogFile.multiplier] );
		unitsBox.text := fLogFile.units;
	end;

begin
	fLogFile := aLogFile;
	if aLogFile = nil then initNewLog
	else iniLogProps;
end;


function tNewLogWizard.getMultiplier: double;
var
	s: string;
	v: double;
	op: (opEq, opMul, opDiv);

	procedure doIt( p1, p2: integer );
	var
		p: integer;
		s1: string;
		v1: double;
	begin
		if (p1 > 0) and ((p1 < p2) or (p2 = 0)) then p := p1
		else if (p2 > 0) then p := p2
		else p := succ( length( s ) );
		s1 := trim( copy( s, 1, pred( p ) ) );
		s := trim( copy( s, succ( p ), maxInt ) );
		if (s1 = '') or ((p1 > 0) or (p2 > 0)) and (s = '') then
			v := nan
		else
			begin
			try
				v1 := strToFloat( s1 );
				case op of
					opEq:	v := v1;
					opMul:	v := v * v1;
					else	if v1 = 0 then v := nan
							else v := v / v1;
					end;
				except v := nan;
				end;
			if p = p1 then op := opMul
			else op := opDiv;
			end;
	end;

begin
	s := trim( multiplierBox.text );
	if s = '' then
		result := nan
	else
		begin
		op := opEq;
		v := 1;
		while (s <> '') and not isNAN( v ) do doIt( pos( '*', s ), pos( '/', s ) );
		if isNAN( v ) or (v = 0) then result := nan
		else result := v;
		end;
end;

procedure tNewLogWizard.treeChange(Sender: TObject; Node: TTreeNode);
begin
	okBtn.enabled := validate;
	if node.level = 2 then
		begin
		if pos( '.', node.text ) > 0 then typeBox.itemIndex := 4
		else if typeBox.itemIndex = 4 then typeBox.itemIndex := 3;
		flagsSavedOnChangeBox.checked := (node <> fQueuedClientsNode) and not sameText( node.parent.parent.text, fnStatisticsIni );
		if node = fQueuedClientsNode then
			begin
			typeBox.itemIndex := pred( ord( ltInteger ) );
			typeBox.enabled := false;
			flagsCumulBox.checked := false;
			flagsCumulBox.enabled := false;
			flagsSavedOnChangeBox.checked := false;
			flagsSavedOnChangeBox.enabled := false;
			end
		else
			begin
			typeBox.enabled := (fLogFile = nil);
			flagsCumulBox.enabled := (fLogFile = nil) or (fLogFile.what <> lwQueuedClients);
			flagsSavedOnChangeBox.enabled := (fLogFile = nil);
			end;
		end;
end;

function tNewLogWizard.validate: boolean;
var
	node: tTreeNode;
	key: string;
begin
	node := tree.selected;
	if (fLogFile <> nil) or (node <> nil) and (node.level = 2) then
		begin
		if fLogFile = nil then
			begin
			key := trim( copy( node.text, 1, pred( pos( '=', node.text ) ) ) );
			if node = fQueuedClientsNode then fileName := fnQueueLog
			else fileName := format( iniLogFmt, [node.parent.parent.text, node.parent.text, key] );
			result := not fileExists( pnEFSLogs + fileName + enLogExt )
				and not fileExists( pnEFSLogs + fileName + disLogExt )
				and not fileExists( pnEFSLogs + fileName + permLogExt )
				and not isNAN( multiplier );
(*			result := not fileExists( logsPath + fileName + enLogExt )
				and not fileExists( logsPath + fileName + disLogExt )
				and not fileExists( logsPath + fileName + permLogExt )
				and not isNAN( multiplier );
*)			end
		else
			result := not isNAN( multiplier );
		dataType := tLogType( succ( typeBox.itemIndex ) );
		flags := 0;
		if flagsCumulBox.checked then flags := flags or lfCumulativeValue;
		if flagsSavedOnChangeBox.checked then flags := flags or lfSavedOnChange;
		if flagsShowWithRequestsBox.checked then flags := flags or lfShowWithRequests;
		units := unitsBox.text;
		end
	else
		result := false;
end;

procedure tNewLogWizard.multiplierBoxChange(Sender: TObject);
begin
	okBtn.enabled := validate;
end;

procedure tNewLogWizard.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
	canClose := (modalResult = mrCancel) or validate;
end;

procedure tNewLogWizard.appEventsShowHint(var HintStr: String; var CanShow: Boolean; var HintInfo: THintInfo);
begin
	hintStr := stringReplace( stringReplace( stringReplace( stringReplace( hintStr, '\\', #1, [rfReplaceAll] ), '\n', #13#10, [rfReplaceAll] ), #1, '\', [rfReplaceAll] ), ';   ', #13#10, [rfReplaceAll] );
end;

procedure tNewLogWizard.FormCreate(Sender: TObject);
begin
	hintDelay := application.hintHidePause;
	application.hintHidePause := 10000;
end;

procedure tNewLogWizard.FormDestroy(Sender: TObject);
begin
	application.hintHidePause := hintDelay;
end;

end.
