{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit repairKMUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ActnList, Buttons, strUtils, StdActns;

resourceString                                   
	rsImportFromBackupHlp =
		  'This option can be used before eMule has rehashed all your shared files (and may prevent eMule from rehashing some of your files)'#13#10
		+ 'This will simply copy to known.met the data of each known file which is found in the backup copy but not in known.met.';
	rsImportFromLinksHlp =
		  'This option can be used before eMule has rehashed all your shared files (and will prevent eMule from rehashing the imported files)'#13#10
		+ 'Obviously, this option can only restore what is included if ed2k links, that is: files name, size, hash and, if provided, AICH hash.';
	rsRecalculateStatsHlp =
		  'This option should only be used AFTER EMULE HAS REHASHED YOUR FILES.'#13#10
		+ 'This will analyse your EFS logs to recalculate the known files stats (total transferred/requests/accepted requests). Your EFS logs will be updated accordingly.'#13#10
		+ 'This option alone will only recalculate the stats for the files found in both known.met and EFS logs.';
	rsImportFromLogsHlp =
		  'If this option is selected, the files which are no longer found in known.met will be restored as well, but since files names and sizes are not stored in EFS logs, they will all be entitled "Unknown filename", and their size will be set to 0.'#13#10
		+ 'You can afterward use one of the two first options to restore their names and sizes.';
	rsDeleteErroneousHlp =
		  'This will remove any known file with either an empty filename (or no filename at all), or a hash = 00000000000000000000000000000000';
	rsCannotEditKMWhenEmuleIsRunning =
		  'Known.met cannot be modified while eMule is running';
	rsCannotEditKMIfModified =
		  'Known.met has been modified while this dialog was open. Please reload the known files (from the main window) and try again.';
	rsCannotImportFromSelf =
		  'Please select a backup copy of known.met...';

type
  tRepairKMForm = class(TForm)
    topPanel: TPanel;
    bottomPanel: TPanel;
    okBtn: TButton;
    cancelBtn: TButton;
    ActionList1: TActionList;
    helpBox: TMemo;
	importFromBackupBox: TCheckBox;
	importFromLinksBox: TCheckBox;
	recalculateStatsBox: TCheckBox;
    importFromLogsBox: TCheckBox;
	selectSrcBackupBtn: TButton;
	ed2kLinksBox: TMemo;
    aBackupKM: TAction;
    aSelectKMBackup: TFileOpen;
    deleteErroneousBox: TCheckBox;
    SpeedButton1: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure aBackupKMExecute(Sender: TObject);
    procedure aSelectKMBackupAccept(Sender: TObject);
    procedure ed2kLinksBoxChange(Sender: TObject);
	procedure recalculateStatsBoxClick(Sender: TObject);
    procedure noHelpEnter(Sender: TObject);
    procedure importFromBackupEnter(Sender: TObject);
    procedure importFromLinksEnter(Sender: TObject);
    procedure recalculateStatsBoxEnter(Sender: TObject);
    procedure importFromLogsBoxEnter(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure deleteErroneousBoxEnter(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
	backupKMCaption,
	selectKMBackupCaption: string;
	function getImportFrom: string;
	function getImportLinks: tStrings;
  public
	property importFrom: string read getImportFrom;
	property importLinks: tStrings read getImportLinks;
  end;

var
  repairKMForm: tRepairKMForm;

implementation

{$R *.dfm}

uses
	mainUnit, basicDefsUnit;

procedure tRepairKMForm.FormCreate(Sender: TObject);
begin
	backupKMCaption := aBackupKM.caption;
	selectKMBackupCaption := aSelectKMBackup.caption;
	if backupKnownMetDone then aBackupKM.caption := backupKMCaption + ' (' + rsDone + ')';
end;

procedure tRepairKMForm.aBackupKMExecute(Sender: TObject);
var
	reply: integer;
begin
	mainForm.backupMet( reply );
	if reply <> idCancel then aBackupKM.caption := backupKMCaption + ' (' + rsDone + ')';
end;

procedure tRepairKMForm.aSelectKMBackupAccept(Sender: TObject);
begin
	if sameText( knownMet.path, aSelectKMBackup.dialog.fileName ) then
		begin
		application.messageBox( pChar( rsCannotImportFromSelf ), pChar( appFullName ), mb_ok or mb_iconError );
		aSelectKMBackup.caption := selectKMBackupCaption;
		importFromBackupBox.enabled := false;
		importFromBackupBox.checked := false;
		end
	else
		begin
		aSelectKMBackup.caption := aSelectKMBackup.dialog.fileName;
		importFromBackupBox.enabled := true;
		importFromBackupBox.checked := true;
		end;
end;

procedure tRepairKMForm.ed2kLinksBoxChange(Sender: TObject);

	function validateLinks: boolean;

		function isValidLink( const s: string ): boolean;
		var
			p1, p2, i: integer;
			ss: string;
		begin
			result := false;
			p1 := pos( '|', s );													{ ed2k://| }
			if p1 = 0 then exit;
			p2 := posEx( '|', s, succ( p1 ) );										{ ...file| }
			if (p2 = 0) or (copy( s, 1, pred( p2 ) ) <> 'ed2k://|file') then exit;
			p1 := posEx( '|', s, succ( p2 ) );										{ ...<filename>| }
			if p1 = 0 then exit;
			p2 := posEx( '|', s, succ( p1 ) );										{ ...<filesize>| }
			if (p2 = 0) or (p2 = succ( p1 )) then exit;
			try
				strToInt64( copy( s, succ( p1 ), pred( p2 - p1 ) ) );
				except exit;
				end;
			p1 := posEx( '|', s, succ( p2 ) );										{ ...<hash>| }
			{ 08 may 07: bug fix: Import missing known files from ed2k &links -> support for links with AICH >>> }
			if (p1 = 0) or (pred( p1 - p2 ) <> 32) then exit;
			for i := succ( p2 ) to p2 + 32 do
				if not (upCase( s[ i ] ) in ['0'..'9', 'A'..'F']) then
					exit;
			p2 := posEx( '|', s, succ( p1 ) );
			if (p2 = 0) then														{ .../ }
				begin
				if copy( s, succ( p1 ), maxInt ) <> '/' then exit;
				end
			else																	{ ...h=<AICH>| }
				begin
				ss := copy( s, succ( p1 ), 2 );
				if ss = '' then ;
				ss := copy( s, succ( p2 ), maxInt );
				if ss = '' then ;
				if (p2 <> p1 + 35) or (copy( s, succ( p1 ), 2 ) <> 'h=') or (copy( s, succ( p2 ), maxInt ) <> '/') then exit;
				for i := p1 + 3 to p1 + 34 do
					if not (upCase( s[ i ] ) in ['0'..'9', 'A'..'Z']) then
						exit;
				end;
{ old code:
			if (p1 = 0) or (pred( p1 - p2 ) <> 32) or (copy( s, succ( p1 ), maxInt ) <> '/') then exit;
			for p1 := succ( p2 ) to p2 + 32 do
				if not (upCase( s[ p1 ] ) in ['0'..'9', 'A'..'F']) then
					exit;
}			{ 08 may 07: bug fix: Import missing known files from ed2k &links -> support for links with AICH >>> }
			result := true;
		end;

	var
		i: integer;
	begin
		result := false;
		for i := 0 to pred( ed2kLinksBox.lines.count ) do if ed2kLinksBox.lines[ i ] <> '' then
			begin
			if isValidLink( ed2kLinksBox.lines[ i ] ) then
				result := true
			else
				begin
				result := false;
				exit;
				end;
			end;
	end;

begin
	importFromLinksBox.enabled := validateLinks;
	importFromLinksBox.checked := importFromLinksBox.enabled;
end;

procedure tRepairKMForm.recalculateStatsBoxClick(Sender: TObject);
begin
	importFromLogsBox.enabled := recalculateStatsBox.checked;
	importFromLogsBox.checked := recalculateStatsBox.checked;
end;

function tRepairKMForm.getImportFrom: string;
begin
	result := aSelectKMBackup.caption;
end;

function tRepairKMForm.getImportLinks: tStrings;
begin
	result := ed2kLinksBox.lines;
end;

procedure tRepairKMForm.noHelpEnter(Sender: TObject);
begin
	helpBox.Lines.text := '';
end;

procedure tRepairKMForm.importFromBackupEnter(Sender: TObject);
begin
	helpBox.Lines.text := rsImportFromBackupHlp;
end;

procedure tRepairKMForm.importFromLinksEnter(Sender: TObject);
begin
	helpBox.Lines.text := rsImportFromLinksHlp;
end;

procedure tRepairKMForm.recalculateStatsBoxEnter(Sender: TObject);
begin
	helpBox.Lines.text := rsRecalculateStatsHlp;
end;

procedure tRepairKMForm.importFromLogsBoxEnter(Sender: TObject);
begin
	helpBox.Lines.text := rsImportFromLogsHlp;
end;

procedure tRepairKMForm.deleteErroneousBoxEnter(Sender: TObject);
begin
	helpBox.Lines.text := rsDeleteErroneousHlp;
end;

procedure tRepairKMForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
	{ 08 may 07: Bug fix: disable known.met editing features if known.met has been modified since last reload >>> }
	if (modalResult = mrOk) and canClose and (mainForm.isEmuleRunning or not knownMet.isUpTodate) then
{ old code:
	if (modalResult = mrOk) and canClose and mainForm.isEmuleRunning then
}	{ <<< 08 may 07: Bug fix: disable known.met editing features if known.met has been modified since last reload }
		begin
		canClose := false;
		{ 08 may 07: Bug fix: disable known.met editing features if known.met has been modified since last reload >>> }
		if mainForm.isEmuleRunning then application.messageBox( pChar( rsCannotEditKMWhenEmuleIsRunning ), pChar( appFullName ), mb_ok or mb_iconError )
		else application.messageBox( pChar( rsCannotEditKMIfModified ), pChar( appFullName ), mb_ok or mb_iconError );
{ old code:
		application.messageBox( pChar( rsCannotEditKMWhenEmuleIsRunning ), pChar( appFullName ), mb_ok or mb_iconError );
}		{ <<< 08 may 07: Bug fix: disable known.met editing features if known.met has been modified since last reload }
		end;
end;

procedure tRepairKMForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
	if key = #27 then modalResult := mrCancel;
end;

end.
