{
Copyright (C)2006 JesTheOnlyOne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

{$WARN SYMBOL_PLATFORM OFF}
{$IFDEF debug}
	{$R+,I+,Q+}
{$ELSE}
	{$R-,I-,Q-}
{$ENDIF}
unit tipsUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

resourceString
	tips1 =
		  'Right-click on a column title to reverse its sort order without changing the columns priority.';
	tips2 =
		  'You can use the Shift and Ctrl keys to select multiple items.\n\n'
		+ 'When several items are selected, pressing space or clicking in a checkbox will activate / unactivate the logs of all the selected items.';
	tips3 =
		  'You can use the two scrollbars at the bottom to zoom and/or scroll the graphics.\n'
		+ 'Alternatively, you can shift-click / ctrl-click on the graphs to zoom / unzoom and center on the clicked point.';
	tips4 =
		  'Don''t expect to see nice graphics at once:\n\n'
		+ '- The logs are updated every 11 minutes (when eMule saves known.met).\n'
		+ '- Due to the way the graphs are computed, at least 4 values are needed before anything is shown on the graphs (that is, after 44 minutes).\n'
		+ '- And anyway, to get useful and significant results, you will have to wait at least several hours... if not several days.';
	tips5 =
		  'If your log files are becoming too large, you can use the menu "%s" -> "%s" to reduce their size.\n\n'
		+ 'This will delete some of the recorded values in every log file, but only in parts older than %d days (the older, the more compressed).\n\n'
		+ 'In the oldest parts, at least one value every %d hours will be kept.';
	tips6 =
		  'You can log any numeric value from any .ini file located the eMule\config directory.\n'
		+ 'Use the menu "%s" -> "%s" to add new logs.';
	tips7 =
		  'The "DLed" and "DL time" columns are only indicative.\n\n'
		+ 'In the "DLed" column:\n'
		+ '- "%s" means that the file has probably been copied from a CD or another drive (the creation date is more recent than the last modification date)\n'
		+ '- "%s" means that less than %d seconds ellapsed between the creation and the last modification of the file (which is very fast for an emule download...)\n'
		+ '- "%s" means that there''s no way to guess if the file has been downloaded or not\n'
		+ '- "%s" just means %s (there''s still a PARTFILENAME tag for this file)\n\n'
		+ 'The "DL time" column shows the difference between the last modification date and the creation date of the file. This is only meaningful for files that have actually been downloaded, and never modified since completion.';
	tips8 =
		  'Double click on a graphic to add a special event at a given date.\n'
		+ 'Examples:\n'
		+ '07/08/2006 = new ISP\n'
		+ '09/20/2006 = Enabled obfuscation\n\n'
		+ 'An event can be associated to a specific file.\n'
		+ 'Example:\n'
		+ '10/15/2006 = changed from release to auto priority';
	tips9 =
		  'You can rename unshared known files, or set them to unknown or cancelled state.\n'
		+ 'See the "Known files" -> "Edit known.met" menu.\n\n'
		+ 'Note that those features are disabled if:\n'
		+ '- EMule is running\n'
		+ '- At least one of the selected files is still shared\n'
		+ '- Known.met has been modified since you have started EFS or reloaded the known files list.';
	tips10 =
		  'If the EFS icon in the notification area (AKA sysTray) shows a red ! this means that the recorder encountered an error.\n'
		+ 'Each recorder''s error is saved in a log which can be viewed from the viewer''s file menu.';
	tips11 =
		  'It''s possible to combine two existing eMule stats logs into a single graph.\n\n'
		+ 'For example, to create a graph for the "Average uploaded per session" stat:\n'
		+ '- Select both the "TotalUploadedBytes" and "UpSuccessfulSessions" logs\n'
		+ '- In the context menu, choose "Combine logs..."\n'
		+ '- Select the division operation.';

type
  tTipsForm = class(TForm)
	topPanel: TPanel;
	bottomPanel: TPanel;                                             
	closeBtn: TButton;
	prevBtn: TButton;
	nextBtn: TButton;
	tipsBox: TMemo;
    showAtStartupBox: TCheckBox;
    bugsBtn: TButton;
	procedure FormCreate(Sender: TObject);
	procedure FormDestroy(Sender: TObject);
	procedure nextBtnClick(Sender: TObject);
	procedure prevBtnClick(Sender: TObject);
	procedure showTips;
	procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
  public
	tipsList: tStringList;
	tipsIndex: integer;
  end;

var
  tipsForm: tTipsForm;

implementation

uses mainUnit;

{$R *.dfm}

procedure tTipsForm.FormCreate(Sender: TObject);
begin
	tipsList := tStringList.create;
	tipsList.sorted := false;
	tipsList.caseSensitive := false;
	tipsList.duplicates := dupIgnore;
	tipsList.add( tips1 );
	tipsList.add( tips2 );
	tipsList.add( tips3 );
	tipsList.add( tips4 );
	tipsList.add( stringReplace( format( tips5, [mainForm.fileMenu.caption, mainForm.aFileCompressLogs.caption, daysCompressThreshold, hoursCompressMax] ), '&', '', [rfReplaceAll] ) );
	tipsList.add( stringReplace( format( tips6, [mainForm.iniFilesMenu.caption, mainForm.aIniNewLog.caption] ), '&', '', [rfReplaceAll] ) );
	tipsList.add( format( tips7, [rsNo, rsMaybeNo, minDLSecs, rsMaybeYes, rsYes, rsYes] ) );
	tipsList.add( tips8 );
	tipsList.add( tips9 );
	tipsList.add( tips10 );
	tipsList.add( tips11 );
	tipsIndex := random( tipsList.count );
	showTips;
end;

procedure tTipsForm.FormDestroy(Sender: TObject);
begin
	tipsList.free;
end;

procedure tTipsForm.nextBtnClick(Sender: TObject);
begin
	if tipsIndex < pred( tipsList.count ) then inc( tipsIndex ) else tipsIndex := 0;
	showTips;
end;

procedure tTipsForm.prevBtnClick(Sender: TObject);
begin
	if tipsIndex > 0 then dec( tipsIndex ) else tipsIndex := pred( tipsList.count );
	showTips;
end;

procedure tTipsForm.showTips;
begin
	tipsBox.lines.text := stringReplace( tipsList[ tipsIndex ], '\n', #13, [rfReplaceAll] );
end;

procedure tTipsForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
	if (key = #13) and tipsBox.focused then nextBtn.click
	else if key = #27 then modalResult := mrCancel; 
end;

end.
